@@ -32,8 +32,8 @@ from oslo.config import cfg
 from oslo.messaging._drivers import base
 from oslo.messaging._drivers import common as rpc_common
 from oslo.messaging._executors import impl_eventlet  # FIXME(markmc)
-from oslo.messaging.openstack.common.gettextutils import _
-from oslo.messaging.openstack.common import jsonutils
+from oslo.messaging._i18n import _
+from oslo.serialization import jsonutils
 from oslo.utils import excutils
 from oslo.utils import importutils
 
@@ -88,7 +88,6 @@ zmq_opts = [
 
 CONF = cfg.CONF
 
-ZMQ_CTX = None  # ZeroMQ Context, must be global.
 matchmaker = None  # memoized matchmaker object
 
 
@@ -119,7 +118,8 @@ class ZmqSocket(object):
     """
 
     def __init__(self, addr, zmq_type, bind=True, subscribe=None):
-        self.sock = _get_ctxt().socket(zmq_type)
+        self.ctxt = zmq.Context(CONF.rpc_zmq_contexts)
+        self.sock = self.ctxt.socket(zmq_type)
         self.addr = addr
         self.type = zmq_type
         self.subscriptions = []
@@ -196,6 +196,7 @@ class ZmqSocket(object):
         try:
             # Default is to linger
             self.sock.close()
+            self.ctxt.term()
         except Exception:
             # While this is a bad thing to happen,
             # it would be much worse if some of the code calling this
@@ -759,67 +760,6 @@ def _multi_send(method, context, topic, msg, timeout=None,
     return return_val
 
 
-def create_connection(conf, new=True):
-    return Connection(conf)
-
-
-def multicall(conf, *args, **kwargs):
-    """Multiple calls."""
-    return _multi_send(_call, *args, **kwargs)
-
-
-def call(conf, *args, **kwargs):
-    """Send a message, expect a response."""
-    data = _multi_send(_call, *args, **kwargs)
-    return data[-1]
-
-
-def cast(conf, *args, **kwargs):
-    """Send a message expecting no reply."""
-    _multi_send(_cast, *args, **kwargs)
-
-
-def fanout_cast(conf, context, topic, msg, **kwargs):
-    """Send a message to all listening and expect no reply."""
-    # NOTE(ewindisch): fanout~ is used because it avoid splitting on .
-    # and acts as a non-subtle hint to the matchmaker and ZmqProxy.
-    _multi_send(_cast, context, 'fanout~' + six.text_type(topic),
-                msg, **kwargs)
-
-
-def notify(conf, context, topic, msg, envelope):
-    """Send notification event.
-
-    Notifications are sent to topic-priority.
-    This differs from the AMQP drivers which send to topic.priority.
-    """
-    # NOTE(ewindisch): dot-priority in rpc notifier does not
-    # work with our assumptions.
-    topic = topic.replace('.', '-')
-    cast(conf, context, topic, msg, envelope=envelope)
-
-
-def cleanup():
-    """Clean up resources in use by implementation."""
-    global ZMQ_CTX
-    if ZMQ_CTX:
-        ZMQ_CTX.term()
-    ZMQ_CTX = None
-
-    global matchmaker
-    matchmaker = None
-
-
-def _get_ctxt():
-    if not zmq:
-        raise ImportError("Failed to import eventlet.green.zmq")
-
-    global ZMQ_CTX
-    if not ZMQ_CTX:
-        ZMQ_CTX = zmq.Context(CONF.rpc_zmq_contexts)
-    return ZMQ_CTX
-
-
 def _get_matchmaker(*args, **kwargs):
     global matchmaker
     if not matchmaker:
@@ -898,6 +838,8 @@ class ZmqDriver(base.BaseDriver):
 
     def __init__(self, conf, url, default_exchange=None,
                  allowed_remote_exmods=None):
+        if not zmq:
+            raise ImportError("Failed to import eventlet.green.zmq")
         conf.register_opts(zmq_opts)
         conf.register_opts(impl_eventlet._eventlet_opts)
 
@@ -939,6 +881,8 @@ class ZmqDriver(base.BaseDriver):
             # NOTE(ewindisch): fanout~ is used because it avoid splitting on
             # and acts as a non-subtle hint to the matchmaker and ZmqProxy.
             topic = 'fanout~' + topic
+        elif target.server:
+            topic = '%s.%s' % (topic, target.server)
 
         reply = _multi_send(method, context, topic, message,
                             envelope=envelope,
@@ -962,7 +906,7 @@ class ZmqDriver(base.BaseDriver):
         return self._send(target, ctxt, message, envelope=(version == 2.0))
 
     def listen(self, target):
-        conn = create_connection(self.conf)
+        conn = Connection(self.conf)
 
         listener = ZmqListener(self)
 
@@ -975,12 +919,14 @@ class ZmqDriver(base.BaseDriver):
 
         return listener
 
-    def listen_for_notifications(self, targets_and_priorities):
+    def listen_for_notifications(self, targets_and_priorities, pool):
         # NOTE(sileht): this listener implementation is limited
-        # because zeromq doesn't support requeing message
-        conn = create_connection(self.conf)
+        # because zeromq doesn't support:
+        #  * requeing message
+        #  * pool
+        conn = Connection(self.conf)
 
-        listener = ZmqListener(self, None)
+        listener = ZmqListener(self)
         for target, priority in targets_and_priorities:
             # NOTE(ewindisch): dot-priority in rpc notifier does not
             # work with our assumptions.
@@ -992,4 +938,4 @@ class ZmqDriver(base.BaseDriver):
         return listener
 
     def cleanup(self):
-        cleanup()
+        pass
