@@ -25,65 +25,21 @@ functions scheduled by the Controller.
 """
 
 import abc
-import collections
 import logging
 import threading
 import uuid
 
+import proton
 import pyngus
 from six import moves
 
 from oslo.config import cfg
 from oslo.messaging._drivers.protocols.amqp import eventloop
+from oslo.messaging._drivers.protocols.amqp import opts
+from oslo.messaging import transport
 
 LOG = logging.getLogger(__name__)
 
-_amqp1_opts = [
-    cfg.StrOpt('server_request_prefix',
-               default='exclusive',
-               help="address prefix used when sending to a specific server"),
-
-    cfg.StrOpt('broadcast_prefix',
-               default='broadcast',
-               help="address prefix used when broadcasting to all servers"),
-
-    cfg.StrOpt('group_request_prefix',
-               default='unicast',
-               help="address prefix when sending to any server in group"),
-
-    cfg.StrOpt('container_name',
-               default=None,
-               help='Name for the AMQP container'),
-
-    cfg.IntOpt('idle_timeout',
-               default=0,  # disabled
-               help='Timeout for inactive connections (in seconds)'),
-
-    cfg.BoolOpt('trace',
-                default=False,
-                help='Debug: dump AMQP frames to stdout'),
-
-    cfg.StrOpt('ssl_ca_file',
-               default='',
-               help="CA certificate PEM file for verifing server certificate"),
-
-    cfg.StrOpt('ssl_cert_file',
-               default='',
-               help='Identifying certificate PEM file to present to clients'),
-
-    cfg.StrOpt('ssl_key_file',
-               default='',
-               help='Private key PEM file used to sign cert_file certificate'),
-
-    cfg.StrOpt('ssl_key_password',
-               default=None,
-               help='Password for decrypting ssl_key_file (if encrypted)'),
-
-    cfg.BoolOpt('allow_insecure_clients',
-                default=False,
-                help='Accept clients using either SSL or plain TCP')
-]
-
 
 class Task(object):
     """Perform a messaging operation via the Controller."""
@@ -101,8 +57,11 @@ class Replies(pyngus.ReceiverEventHandler):
         self._correlation = {}  # map of correlation-id to response queue
         self._ready = False
         self._on_ready = on_ready
+        rname = "Consumer-%s:src=[dynamic]:tgt=replies" % uuid.uuid4().hex
         self._receiver = connection.create_receiver("replies",
-                                                    event_handler=self)
+                                                    event_handler=self,
+                                                    name=rname)
+
         # capacity determines the maximum number of reply messages this link
         # can receive. As messages are received and credit is consumed, this
         # driver will 'top up' the credit back to max capacity.  This number
@@ -233,25 +192,24 @@ class Server(pyngus.ReceiverEventHandler):
 
 
 class Hosts(object):
-    """An order list of peer addresses.  Connection failover progresses from
-    one host to the next.
+    """An order list of TransportHost addresses. Connection failover
+    progresses from one host to the next.
     """
-    HostnamePort = collections.namedtuple('HostnamePort',
-                                          ['hostname', 'port'])
-
     def __init__(self, entries=None):
-        self._entries = [self.HostnamePort(h, p) for h, p in entries or []]
+        self._entries = entries[:] if entries else []
+        for entry in self._entries:
+            entry.port = entry.port or 5672
         self._current = 0
 
-    def add(self, hostname, port=5672):
-        self._entries.append(self.HostnamePort(hostname, port))
+    def add(self, transport_host):
+        self._entries.append(transport_host)
 
     @property
     def current(self):
         if len(self._entries):
             return self._entries[self._current]
         else:
-            return self.HostnamePort("localhost", 5672)
+            return transport.TransportHost(hostname="localhost", port=5672)
 
     def next(self):
         if len(self._entries) > 1:
@@ -262,7 +220,7 @@ class Hosts(object):
         return '<Hosts ' + str(self) + '>'
 
     def __str__(self):
-        return ", ".join(["%s:%i" % e for e in self._entries])
+        return ", ".join(["%r" % th for th in self._entries])
 
 
 class Controller(pyngus.ConnectionEventHandler):
@@ -287,24 +245,25 @@ class Controller(pyngus.ConnectionEventHandler):
         self._servers = {}
         self.hosts = Hosts(hosts)
 
-        opt_group = cfg.OptGroup(name='amqp1',
-                                 title='AMQP 1.0 options')
+        opt_group = cfg.OptGroup(name='oslo_messaging_amqp',
+                                 title='AMQP 1.0 driver options')
         config.register_group(opt_group)
-        config.register_opts(_amqp1_opts, group=opt_group)
+        config.register_opts(opts.amqp1_opts, group=opt_group)
 
-        self.server_request_prefix = config.amqp1.server_request_prefix
-        self.broadcast_prefix = config.amqp1.broadcast_prefix
-        self.group_request_prefix = config.amqp1.group_request_prefix
-        self._container_name = config.amqp1.container_name
-        if not self._container_name:
-            self._container_name = "container-%s" % uuid.uuid4().hex
-        self.idle_timeout = config.amqp1.idle_timeout
-        self.trace_protocol = config.amqp1.trace
-        self.ssl_ca_file = config.amqp1.ssl_ca_file
-        self.ssl_cert_file = config.amqp1.ssl_cert_file
-        self.ssl_key_file = config.amqp1.ssl_key_file
-        self.ssl_key_password = config.amqp1.ssl_key_password
-        self.ssl_allow_insecure = config.amqp1.allow_insecure_clients
+        self.server_request_prefix = \
+            config.oslo_messaging_amqp.server_request_prefix
+        self.broadcast_prefix = config.oslo_messaging_amqp.broadcast_prefix
+        self.group_request_prefix = \
+            config.oslo_messaging_amqp.group_request_prefix
+        self._container_name = config.oslo_messaging_amqp.container_name
+        self.idle_timeout = config.oslo_messaging_amqp.idle_timeout
+        self.trace_protocol = config.oslo_messaging_amqp.trace
+        self.ssl_ca_file = config.oslo_messaging_amqp.ssl_ca_file
+        self.ssl_cert_file = config.oslo_messaging_amqp.ssl_cert_file
+        self.ssl_key_file = config.oslo_messaging_amqp.ssl_key_file
+        self.ssl_key_password = config.oslo_messaging_amqp.ssl_key_password
+        self.ssl_allow_insecure = \
+            config.oslo_messaging_amqp.allow_insecure_clients
         self.separator = "."
         self.fanout_qualifier = "all"
         self.default_exchange = default_exchange
@@ -332,14 +291,13 @@ class Controller(pyngus.ConnectionEventHandler):
         self._tasks.put(task)
         self._schedule_task_processing()
 
-    def destroy(self):
+    def shutdown(self, wait=True, timeout=None):
         """Shutdown the messaging service."""
         if self.processor:
-            self.processor.wakeup(lambda: self._start_shutdown())
-            LOG.info("Waiting for eventloop to exit")
-            self.processor.join()
+            LOG.debug("Waiting for eventloop to exit")
+            self.processor.shutdown(wait, timeout)
             self.processor = None
-        LOG.info("Eventloop exited, driver shut down")
+        LOG.debug("Eventloop exited, driver shut down")
 
     # The remaining methods are reserved to run from the eventloop thread only!
     # They must not be invoked directly!
@@ -436,8 +394,7 @@ class Controller(pyngus.ConnectionEventHandler):
 
     def _do_connect(self):
         """Establish connection and reply subscription on processor thread."""
-        hostname = self.hosts.current.hostname
-        port = self.hosts.current.port
+        host = self.hosts.current
         conn_props = {}
         if self.idle_timeout:
             conn_props["idle-time-out"] = float(self.idle_timeout)
@@ -454,7 +411,7 @@ class Controller(pyngus.ConnectionEventHandler):
                                             self.ssl_key_file,
                                             self.ssl_key_password)
             conn_props["x-ssl-allow-cleartext"] = self.ssl_allow_insecure
-        self._socket_connection = self.processor.connect(hostname, port,
+        self._socket_connection = self.processor.connect(host,
                                                          handler=self,
                                                          properties=conn_props)
         LOG.debug("Connection initiated")
@@ -577,6 +534,17 @@ class Controller(pyngus.ConnectionEventHandler):
                      reason or "no reason given")
             self._socket_connection.connection.close()
 
+    def sasl_done(self, connection, pn_sasl, outcome):
+        """This is a Pyngus callback invoked by Pyngus when the SASL handshake
+        has completed.  The outcome of the handshake will be OK on success or
+        AUTH on failure.
+        """
+        if outcome == proton.SASL.AUTH:
+            LOG.error("Unable to connect to %s:%s, authentication failure.",
+                      self.hosts.current.hostname, self.hosts.current.port)
+            # requires user intervention, treat it like a connection failure:
+            self._handle_connection_loss()
+
     def _complete_shutdown(self):
         """The AMQP Connection has closed, and the driver shutdown is complete.
         Clean up controller resources and exit.
@@ -616,6 +584,6 @@ class Controller(pyngus.ConnectionEventHandler):
             self._reconnecting = False
             self._senders = {}
             self._socket_connection.reset()
-            hostname, port = self.hosts.next()
-            LOG.info("Reconnecting to: %s:%i", hostname, port)
-            self._socket_connection.connect(hostname, port)
+            host = self.hosts.next()
+            LOG.info("Reconnecting to: %s:%i", host.hostname, host.port)
+            self._socket_connection.connect(host)
