@@ -17,7 +17,8 @@ __all__ = ['Notifier',
            'LoggingNotificationHandler',
            'get_notification_listener',
            'NotificationResult',
-           'PublishErrorsHandler']
+           'PublishErrorsHandler',
+           'LoggingErrorNotificationHandler']
 
 from .notifier import *
 from .listener import *
